# -*- cperl -*-
package mIRCColor;
use strict;
use warnings;

# 0x16 : reverse
# 0x1f : underline
# 0x02 : bold
# 0x03 : color
# 0x0f : original

our @COLOR = (
    '#cdcdcd', # 0  white
    '#000000', # 1  black
    '#3636b3', # 2  blue
    '#2a8d2a', # 3  green
    '#c43b3b', # 4  red
    '#c83232', # 5  light red
    '#802680', # 6  purple
    '#66361f', # 7  orange
    '#daa741', # 8  yellow
    '#3dcd3d', # 9  green
    '#195555', # 10 aqua
    '#2f8d75', # 11 light aqua
    '#4545e6', # 12 blue
    '#b137b1', # 13 light purple
    '#4c4c4c', # 14 grey
    '#969696', # 15 light grey

    '#cdcdcd', # white
    '#000000', # black
    '#3636b3', # blue
    '#2a8d2a', # green
    '#c43b3b', # red
    '#c83232', # light red
    '#802680', # purple
    '#66361f', # orange
    '#daa741', # yellow
    '#3dcd3d', # green
    '#195555', # aqua
    '#2f8d75', # light aqua
    '#4545e6', # blue
    '#b137b1', # light purple
    '#4c4c4c', # grey
    '#969696', # light grey
   );

foreach (<>) {
    chomp;
    print replace($_), "\n";
}

sub replace {
    # mIRCカラーシーケンスを<span style="...">...</span>に變換する。
    my $str = shift;

    # 變換器はState Machineである。
    # 以下は初期状態
    my $forecolor = undef; # 値が存在するなら '#xxxxxx' 形式
    my $backcolor = undef;
    my $bold = 0;
    my $underline = 0;
    my $need_span = 0;
    my $result = '';
    foreach my $part (split m/([\x16\x1f\x02\x0f])|(\x03\d{0,2}(?:,\d{1,2})?)/, $str) {
	defined $part or next;
	length $part or next;
	
	if ($part eq "\x16") {
	    ($backcolor, $forecolor) = (
		defined $forecolor ? $forecolor : 'black',
		defined $backcolor ? $backcolor : 'white',
	       );
	    $need_span = 1;
	}
	elsif ($part eq "\x1f") {
	    $underline = !$underline;
	    $need_span = 1;
	}
	elsif ($part eq "\x02") {
	    $bold = !$bold;
	    $need_span = 1;
	}
	elsif ($part =~ m/^\x03(\d{0,2})(?:,(\d{1,2}))?$/) {
	    my ($fore, $back) = ($1, $2);
	    
	    if (defined $back) {
		$forecolor = $COLOR[$fore];
		$backcolor = $COLOR[$back];
	    }
	    elsif (defined $fore and length $fore) {
		$forecolor = $COLOR[$fore];
	    }
	    else {
		$forecolor = $backcolor = undef;
	    }
	    
	    $need_span = 1;
	}
	elsif ($part eq "\x0f") {
	    $need_span = undef;
	}
	else {
	    if ($need_span) {
		my @style;
		defined $forecolor and
		  push @style, sprintf 'color: %s', $forecolor;
		defined $backcolor and
		  push @style, sprintf 'background-color: %s', $backcolor;
		$bold and
		  push @style, sprintf 'font-weight: bolder';
		$underline and
		  push @style, 'text-decoration: underline';

		$result .= sprintf(
		    '<span style="%s">%s</span>',
		    join('; ', @style),
		    $part);
	    }
	    else {
		$result .= $part;
	    }
	}
    }

    $result;
}

1;
