#!/usr/local/bin/perl
# -*- cperl -*-
use strict;
use warnings;
use lib 'classes';
use CGI;
use CGI::Carp qw(fatalsToBrowser);
use HTML::Template;
use IndexFile;
use ConfigFile;
use LogFile;
use mIRCColor;
use Archive::Tar;
main(CGI->new);

sub main {
    my $q = shift;
    # 以下は必須
    # showlog.cgi/NanminGoya/
    # showlog.cgi/NanminGoya/200401/
    # showlog.cgi/NanminGoya/200401/2004.01.01.html
    # showlog.cgi/?mode=js
    # showlog.cgi/logs.tar.gz
    # 以下は余力があれば
    # showlog.cgi/NanminGoya/200401/2004.01.01.txt
    # showlog.cgi/NanminGoya/200401/2004.01.01.xml
    # showlog.cgi/NanminGoya/NanminGoya.zip
    # showlog.cgi/NanminGoya/NanminGoya/200401/NanminGoya-2004-01.zip
    $_ = $q->path_info;
    s|^/||;
    my ($channel, $month, $day) = split m|/|;
    if ($channel and $channel ne '..') {
	if ($channel eq 'logs.tar.gz') {
	    return reply_logs($q);
	}
	
	if ($month and $month ne '..') {
	    if ($day) {
		if ($day =~ m/^(\d{4}\.\d{2}\.\d{2})\.html/) {
		    show_html_log($q, $channel, $month, $1);
		}
		else {
		    show_error($q, "不明なモードです。");
		}
	    }
	    else {
		show_days($q, $channel, $month);
	    }
	}
	else {
	    show_months($q, $channel);
	}
    }
    else {
	show_error($q, "チャンネル名の指定がありません。");
    }
}

sub show_error {
    my ($q, $message) = @_;

    print
      $q->header(
	-type   => 'text/html; charset=UTF-8',
	-status => '500 Internal Server Error',
       ),
      $q->start_html('LogViewer: エラー'),
      $q->h1('エラー'),
      $q->p($message),
      $q->end_html;
}

sub reply_logs {
    my ($q) = @_;
    my $targz = `tar cf - logs | gzip`;

    print $q->header(
	-type => 'application/gzip',
       );
    print $targz;
}

sub printable_channel_name {
    my $channel = shift;
    map {
	$_->{CHANNEL};
    } grep {
	$_->{DIRECTORY} eq $channel;
    } ConfigFile->logs
}

sub show_months {
    my ($q, $channel) = @_;

    my $t = HTML::Template->new(
	filename => 'tmpl/months.html');

    expand_channel_list($q, $t, $channel);
    expand_date_list($q, $t, $channel);

    $t->param(channel => printable_channel_name($channel));
    $t->param(baseurl => $q->url.'/');
    print $q->header(
	-type => 'text/html; charset=UTF-8');
    print $t->output;
}

sub show_days {
    my ($q, $channel, $month) = @_;

    my $t = HTML::Template->new(
	filename => 'tmpl/days.html');

    expand_channel_list($q, $t, $channel);
    expand_date_list($q, $t, $channel, $month);

    $t->param(channel => printable_channel_name($channel));
    $t->param(baseurl => $q->url.'/');
    print $q->header(
	-type => 'text/html; charset=UTF-8');
    print $t->output;
}

sub expand_channel_list {
    my ($q, $t, $current) = @_;

    my @channels;
    foreach my $ch (ConfigFile->logs) {
	push @channels, {
	    'current-channel' => $ch->{DIRECTORY} eq $current,
	    channeldir  => $ch->{DIRECTORY},
	    channelname => $ch->{CHANNEL},
	};
    }
    $t->param(channels => \@channels);
}

sub expand_date_list {
    my ($q, $t, $channel, $current_month, $current_day) = @_;
    $current_month ||= '';
    $current_day ||= '';

    my @months;
    foreach my $month (reverse LogFile::months($channel)) {
	push @months, {
	    'current-month' => $month eq $current_month,
	    days => do {
		if ($month eq $current_month) {
		    my @days;
		    foreach my $day (reverse LogFile::days($channel, $month)) {
			push @days, {
			    'current-day' => $day eq $current_day,
			    date          => $day,
			    channel       => $channel,
			    month         => $month,
			    privmsgs      => (LogFile::index(
				$channel, $month, $day) || {})->{'PRIVMSG-COUNT'} || 0,
			};
		    }
		    \@days;
		}
		else {
		    [];
		}
	    },
	    channel => $channel,
	    month   => $month,
	    'month-readable' => sprintf(
		'%04d.%02d', $month =~ m/^(\d{4})(\d{2})$/),
	};
    }

    $t->param(months => \@months);
}

sub show_html_log {
    my ($q, $channel, $month, $day) = @_;

    my $t = HTML::Template->new(
	filename => 'tmpl/html_log.html');

    $t->param(channel  => printable_channel_name($channel));
    $t->param(baseurl  => $q->url.'/');
    $t->param(date     => $day);

    my $index = LogFile::index($channel, $month, $day) || {};
    $t->param(privmsgs => $index->{'PRIVMSG-COUNT'} || 0);
    $t->param(notices  => $index->{'NOTICE-COUNT'} || 0);

=pod

    my @per_hour;
    foreach (0 .. 23) {
	push @per_hour, {
	    hour => $_,
	};
    }
    $t->param('per-hour' => \@per_hour);

=cut

    expand_channel_list($q, $t, $channel);
    expand_date_list($q, $t, $channel, $month, $day);

    my $escape = sub {
	$_ = shift;
	
	s/&/&amp;/g;
	s/</&lt;/g;
	s/>/&gt;/g;

	$_;
    };

    my $rawlog = LogFile::content($channel, $month, $day);
    my @log;
    my $re_time = qr/\d\d:\d\d(?::\d\d)?/;
    foreach (split /\r?\n/, $rawlog) {
	if (/^($re_time) <(.+?)> (.+)$/) {
	    # privmsg
	    push @log, sprintf(
		'<span class="time">%s</span> '.
		  '<span class="channel">%s</span> '.
		  '<span class="privmsg">%s</span>',
		$1, $escape->("<$2>"), $escape->($3));
	}
	elsif (/^($re_time) \((.+?)\) (.+)$/) {
	    # notice
	    push @log, sprintf(
		'<span class="time">%s</span> '.
		  '<span class="channel">%s</span> '.
		  '<span class="notice">%s</span>',
		$1, $escape->("($2)"), $escape->($3));
	}
	elsif (/^($re_time) (\+ join .+)$/) {
	    # join
	    push @log, sprintf(
		'<span class="time">%s</span> '.
		  '<span class="join">%s</span> ',
		$1, $escape->($2));
	}
	elsif (/^($re_time) (\- part .+?) \((.*?)\)$/) {
	    # part
	    push @log, sprintf(
		'<span class="time">%s</span> '.
		  '<span class="part">%s</span> '.
		  '<span class="cmdmsg">%s</span>',
		$1, $escape->($2), $escape->("($3)"));
	}
	elsif (/^($re_time) (! quit) \((.*?)\)$/) {
	    # quit
	    push @log, sprintf(
		'<span class="time">%s</span> '.
		  '<span class="quit">%s</span> '.
		  '<span class="cmdmsg">%s</span>',
		$1, $escape->($2), $escape->("($3)"));
	}
	elsif (/^($re_time) (topic .+?) (.*)$/) {
	    # topic
	    push @log, sprintf(
		'<span class="time">%s</span> '.
		  '<span class="topic">%s</span> '.
		  '<span class="topicmsg">%s</span>',
		$1, $escape->($2), $escape->($3));
	}
	else {
	    push @log, $escape->($_);
	}
    }

    foreach (@log) {
	$_ = mIRCColor::replace($_);
    }
    
    $t->param(log => join("<br>\n", @log));

    print $q->header(
	-type          => 'text/html; charset=UTF-8',
	#-Last_Modified => scalar(gmtime(
	#    LogFile::last_modified($channel, $month, $day))),
       );
    print $t->output;
}
